#include "ehs/io/RIFF.h"
#include "ehs/io/File.h"

namespace ehs
{
	RIFF::RIFF(const Str_8& filePath)
	{
		File file(filePath, Mode::READ, Disposition::OPEN);
		Serializer<UInt_64> data = file.ReadSerializer_64(Endianness::LE, file.Size());
		file.Release();

		Str_8 riffId = data.ReadStr<Char_8, UInt_64>(4);
		if (riffId != "RIFF")
		{
			EHS_LOG_INT(LogType::ERR, 0, "File at file path, \"" + filePath + "\", is not a valid RIFF file.");
			return;
		}

		data.SetOffset(data.GetOffset() + 4);

		type = data.ReadStr<Char_8, UInt_64>(4);

		while (data.GetOffset() < data.Size())
		{
			Str_8 id = data.ReadStr<Char_8, UInt_64>(4);
			UInt_32 size = data.Read<UInt_32>();

			chunks.Push(RIFF_Chunk(id, Serializer<UInt_64>(Endianness::LE, &data[data.GetOffset()], size)));

			data.SetOffset(data.GetOffset() + size);
		}
	}

	RIFF::RIFF(Serializer<>& data)
	{
		Str_8 riffId = data.ReadStr<Char_8, UInt_64>(4);
		if (riffId != "RIFF")
		{
			EHS_LOG_INT(LogType::ERR, 0, "Data is not in RIFF format.");
			return;
		}

		data.SetOffset(data.GetOffset() + 4);

		type = data.ReadStr<Char_8, UInt_64>(4);

		while (data.GetOffset() < data.Size())
		{
			Str_8 id = data.ReadStr<Char_8, UInt_64>(4);
			UInt_32 size = data.Read<UInt_32>();

			chunks.Push(RIFF_Chunk(id, Serializer<UInt_64>(Endianness::LE, &data[data.GetOffset()], size)));

			data.SetOffset(data.GetOffset() + size);
		}
	}

	RIFF::operator const RIFF_Chunk*() const
	{
		return &chunks[0];
	}

	Str_8 RIFF::GetType() const
	{
		return type;
	}

	bool RIFF::HasChunk(const UInt_64 hashId) const
	{
		for (UInt_64 i = 0; i < chunks.Size(); ++i)
			if (chunks[i].GetHashId() == hashId)
				return true;

		return false;
	}

	bool RIFF::HasChunk(const Str_8& id) const
	{
		return HasChunk(id.Hash_64());
	}

	RIFF_Chunk RIFF::GetChunk(const UInt_64 hashId) const
	{
		for (UInt_64 i = 0; i < chunks.Size(); ++i)
			if (chunks[i].GetHashId() == hashId)
				return chunks[i];

		return {};
	}

	RIFF_Chunk RIFF::GetChunk(const Str_8& id) const
	{
		return GetChunk(id.Hash_64());
	}
}